@@ -1,3 +1,6 @@
+# Copyright 2008-2009 Kouhei Sutou <kou@cozmixng.org>
+# This file is made available under the same terms as Ruby.
+
 require "rbconfig"
 
 require 'mkmf'
@@ -206,6 +209,7 @@ class PackageConfig
                     "/opt/local/lib/pkgconfig",
                     "/usr/lib64/pkgconfig",
                     "/usr/lib/pkgconfig",
+                    "/usr/X11/lib/pkgconfig/",
                     "/usr/share/pkgconfig"].join(separator)
     libdir = ENV["PKG_CONFIG_LIBDIR"]
     default_path = [libdir, default_path].join(separator) if libdir
