@@ -3,7 +3,7 @@
  * Ruby Cairo Binding
  *
  * $Author: kou $
- * $Date: 2008-06-12 10:59:54 $
+ * $Date: 2008-09-26 13:52:08 $
  *
  * Copyright 2005 Øyvind Kolås <pippin@freedesktop.org>
  * Copyright 2004-2005 MenTaLguY <mental@rydia.com>
@@ -15,10 +15,17 @@
 #include "rb_cairo.h"
 #include "rb_cairo_private.h"
 
+#ifdef HAVE_RUBY_ST_H
+#  include <ruby/st.h>
+#else
+#  include <st.h>
+#endif
+
 VALUE rb_cCairo_Context;
 
 static ID cr_id_surface, cr_id_source;
 static ID cr_id_plus, cr_id_minus, cr_id_multi, cr_id_div;
+static cairo_user_data_key_t cr_object_holder_key;
 
 #define _SELF  (RVAL2CRCONTEXT(self))
 
@@ -57,9 +64,23 @@ rb_cairo_context_from_ruby_object (VALUE obj)
       rb_raise (rb_eTypeError, "not a cairo graphics context");
     }
   Data_Get_Struct (obj, cairo_t, context);
+  if (!context)
+    rb_cairo_check_status (CAIRO_STATUS_NULL_POINTER);
   return context;
 }
 
+static rb_cairo__object_holder_t *
+cr_object_holder_new (VALUE object)
+{
+  return rb_cairo__object_holder_new (rb_cCairo_Context, object);
+}
+
+static void
+cr_object_holder_free (void *ptr)
+{
+  rb_cairo__object_holder_free (rb_cCairo_Context, ptr);
+}
+
 static void
 cr_context_free (void *ptr)
 {
@@ -89,16 +110,57 @@ cr_allocate (VALUE klass)
   return Data_Wrap_Struct (klass, NULL, cr_context_free, NULL);
 }
 
+static void
+cr_set_user_data (cairo_t *cr, const cairo_user_data_key_t *key,
+                  void *user_data, cairo_destroy_func_t destroy)
+{
+#if CAIRO_CHECK_VERSION(1, 4, 0)
+  cairo_set_user_data (cr, key, user_data, destroy);
+#else
+  cairo_surface_t *surface;
+  surface = cairo_get_target (cr);
+  cairo_surface_set_user_data (surface, key, user_data, destroy);
+#endif
+}
+
+static VALUE
+cr_destroy (VALUE self)
+{
+  cairo_t *cr;
+
+  cr = _SELF;
+  cr_set_user_data (cr, &cr_object_holder_key, NULL, NULL);
+  cairo_destroy (cr);
+
+  DATA_PTR (self) = NULL;
+  return Qnil;
+}
+
+static VALUE
+cr_destroy_with_destroy_check (VALUE self)
+{
+  if (DATA_PTR (self))
+    cr_destroy (self);
+  return Qnil;
+}
+
 static VALUE
 cr_initialize (VALUE self, VALUE target)
 {
   cairo_t *cr;
+  VALUE result = Qnil;
 
   cr = cairo_create (RVAL2CRSURFACE (target));
   cr_check_status (cr);
   rb_ivar_set (self, cr_id_surface, target);
+  cr_set_user_data (cr,
+                    &cr_object_holder_key,
+                    cr_object_holder_new(self),
+                    cr_object_holder_free);
   DATA_PTR (self) = cr;
-  return Qnil;
+  if (rb_block_given_p ())
+    result = rb_ensure (rb_yield, self, cr_destroy_with_destroy_check, self);
+  return result;
 }
 
 static VALUE
@@ -318,6 +380,12 @@ cr_set_source_generic (int argc, VALUE *argv, VALUE self)
     {
       return cr_set_source_rgba (argc, argv, self);
     }
+  else if (n == 1 && rb_cairo__is_kind_of (arg1, rb_cCairo_Surface))
+    {
+      return cr_set_source_surface (self, arg1,
+                                    rb_float_new (0),
+                                    rb_float_new (0));
+    }
   else if (n == 1)
     {
       return cr_set_source (self, arg1);
@@ -336,7 +404,7 @@ cr_set_source_generic (int argc, VALUE *argv, VALUE self)
                 "invalid argument (expect "
                 "(red, green, blue), (red, green, blue, alpha), "
                 "([red, green, blue]), ([red, green, blue, alpha]), "
-                "(pattern) or (surface, x, y))");
+                "(surface), (pattern) or (surface, x, y))");
     }
 }
 
@@ -1017,11 +1085,31 @@ cr_clip_rectangle_list (VALUE self)
 static   VALUE
 cr_select_font_face (int argc, VALUE *argv, VALUE self)
 {
-  VALUE family, rb_slant, rb_weight;
+  VALUE rb_family, rb_slant, rb_weight;
+  const char *family;
   cairo_font_slant_t slant;
   cairo_font_weight_t weight;
 
-  rb_scan_args(argc, argv, "12", &family, &rb_slant, &rb_weight);
+  rb_scan_args(argc, argv, "03", &rb_family, &rb_slant, &rb_weight);
+
+  if (NIL_P (rb_family))
+    {
+      family = "";
+    }
+  else if (rb_cairo__is_kind_of (rb_family, rb_cString))
+    {
+      family = RSTRING_PTR (rb_family);
+    }
+  else if (rb_cairo__is_kind_of (rb_family, rb_cSymbol))
+    {
+      family = rb_id2name (SYM2ID (rb_family));
+    }
+  else
+    {
+      rb_raise (rb_eArgError,
+                "family name should be nil, String or Symbol: %s",
+                rb_cairo__inspect (rb_family));
+    }
 
   if (NIL_P (rb_slant))
     slant = CAIRO_FONT_SLANT_NORMAL;
@@ -1033,7 +1121,7 @@ cr_select_font_face (int argc, VALUE *argv, VALUE self)
   else
     weight = RVAL2CRFONTWEIGHT (rb_weight);
 
-  cairo_select_font_face (_SELF, RVAL2CSTR (family), slant, weight);
+  cairo_select_font_face (_SELF, family, slant, weight);
   cr_check_status (_SELF);
   return self;
 }
@@ -1117,6 +1205,40 @@ cr_show_glyphs (VALUE self, VALUE rb_glyphs)
   return self;
 }
 
+#if CAIRO_CHECK_VERSION(1, 8, 0)
+static VALUE
+cr_show_text_glyphs (VALUE self, VALUE rb_utf8, VALUE rb_glyphs,
+                     VALUE rb_clusters, VALUE rb_cluster_flags)
+{
+  cairo_t *cr;
+  const char *utf8;
+  int utf8_len;
+  cairo_glyph_t *glyphs = NULL;
+  int num_glyphs = 0;
+  cairo_text_cluster_t *clusters = NULL;
+  int num_clusters = 0;
+  cairo_text_cluster_flags_t cluster_flags;
+
+  cr = _SELF;
+  utf8 = RSTRING_PTR (rb_utf8);
+  utf8_len = RSTRING_LEN (rb_utf8);
+  rb_cairo__glyphs_from_ruby_object (rb_glyphs, &glyphs, &num_glyphs);
+  rb_cairo__text_clusters_from_ruby_object (rb_clusters,
+                                            &clusters, &num_clusters);
+  cluster_flags = RVAL2CRTEXTCLUSTERFLAGS (rb_cluster_flags);
+  cairo_show_text_glyphs (cr, utf8, utf8_len,
+                          glyphs, num_glyphs,
+                          clusters, num_clusters,
+                          cluster_flags);
+  if (glyphs)
+    cairo_glyph_free (glyphs);
+  if (clusters)
+    cairo_text_cluster_free (clusters);
+
+  return self;
+}
+#endif
+
 static VALUE
 cr_get_font_face (VALUE self)
 {
@@ -1370,6 +1492,21 @@ cr_copy_append_path (VALUE self, VALUE path)
   return self;
 }
 
+static int
+cr_destroy_all_guarded_contexts_at_end_iter (VALUE key, VALUE value, VALUE data)
+{
+  cr_destroy (key);
+  return ST_CONTINUE;
+}
+
+static void
+cr_destroy_all_guarded_contexts_at_end (VALUE data)
+{
+  rb_hash_foreach (rb_cairo__gc_guarded_objects (rb_cCairo_Context),
+                   cr_destroy_all_guarded_contexts_at_end_iter,
+                   Qnil);
+}
+
 void
 Init_cairo_context (void)
 {
@@ -1403,8 +1540,12 @@ Init_cairo_context (void)
 
   rb_define_alloc_func (rb_cCairo_Context, cr_allocate);
 
+  rb_cairo__initialize_gc_guard_holder_class (rb_cCairo_Context);
+  rb_set_end_proc(cr_destroy_all_guarded_contexts_at_end, Qnil);
+
   /* Functions for manipulating state objects */
   rb_define_method (rb_cCairo_Context, "initialize", cr_initialize, 1);
+  rb_define_method (rb_cCairo_Context, "destroy", cr_destroy, 0);
 
   rb_define_method (rb_cCairo_Context, "save", cr_save, 0);
   rb_define_method (rb_cCairo_Context, "restore", cr_restore, 0);
@@ -1507,6 +1648,10 @@ Init_cairo_context (void)
 #endif
   rb_define_method (rb_cCairo_Context, "show_text", cr_show_text, 1);
   rb_define_method (rb_cCairo_Context, "show_glyphs", cr_show_glyphs, 1);
+#if CAIRO_CHECK_VERSION(1, 8, 0)
+  rb_define_method (rb_cCairo_Context, "show_text_glyphs",
+                    cr_show_text_glyphs, 4);
+#endif
   rb_define_method (rb_cCairo_Context, "text_path", cr_text_path, 1);
   rb_define_method (rb_cCairo_Context, "glyph_path", cr_glyph_path, 1);
   rb_define_method (rb_cCairo_Context, "text_extents", cr_text_extents, 1);
