@@ -2,6 +2,8 @@ require 'cairo'
 require 'stringio'
 
 class SurfaceTest < Test::Unit::TestCase
+  include CairoTestUtils
+
   def test_new
     output = StringIO.new
     surface = Cairo::PDFSurface.new(output, 10, 10)
@@ -25,4 +27,15 @@ class SurfaceTest < Test::Unit::TestCase
       end
     end
   end
+
+  def test_fallback_resolution
+    only_cairo_version(1, 7, 2)
+
+    surface = Cairo::ImageSurface.new(100, 100)
+
+    assert_equal([300.0, 300.0], surface.fallback_resolution)
+
+    surface.set_fallback_resolution(95, 95)
+    assert_equal([95.0, 95.0], surface.fallback_resolution)
+  end
 end
